import io
import csv
import json
import codecs
from django.http import JsonResponse


class AjaxHelpers:
    ajax_commands = ['button', 'tooltip', 'timer']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.response_commands = []

    def add_command(self, function_name, **kwargs):
        kwargs['function'] = function_name
        self.response_commands.append(kwargs)

    def command_response(self, function_name=None, **kwargs):
        if function_name is not None:
            self.add_command(function_name, **kwargs)
        return JsonResponse(self.response_commands, safe=False)

    def post(self, request, *args, **kwargs):
        if request.is_ajax() and request.content_type == 'application/json':
            response = json.loads(request.body)
            for t in self.ajax_commands:
                if t in response:
                    return getattr(self, f'{t}_{response[t]}')(**response)
        if hasattr(super(), 'post'):
            return super().post(request, *args, **kwargs)


class ReceiveForm:

    def post(self, request, *args, **kwargs):
        if request.is_ajax() and request.content_type == 'multipart/form-data':
            if 'form_id' in request.POST:
                return getattr(self, f'form_{request.POST["form_id"]}')(**request.POST.dict())
        return super().post(request, *args, **kwargs)


class UTF8EncodedStringIO(io.StringIO):
    def next(self):
        return super(UTF8EncodedStringIO, self).next().encode('utf-8')


class AjaxFileHelpers(AjaxHelpers):

    @staticmethod
    def get_csv(file):
        s = file.read()
        if s.startswith(codecs.BOM_UTF8):
            s = s[3:]
        s = s.decode('latin1')
        stream = UTF8EncodedStringIO(s)
        return csv.DictReader(stream, delimiter=',')

    @staticmethod
    def file_upload(file):
        return JsonResponse({'file': 'ok'})

    def post(self, request, *args, **kwargs):
        if request.is_ajax() and request.content_type == 'multipart/form-data' and 'ajax_file' in request.FILES:
            return self.file_upload(request.FILES['ajax_file'])
        return super().post(request, *args, **kwargs)



import json
from django import template
from django.templatetags.static import static
from django.urls import reverse
from django.utils.safestring import mark_safe
from ..utils import random_string

register = template.Library()


@register.simple_tag
def init_ajax():
    return mark_safe(f'<script src="{static("ajax_helpers/ajax_helpers.js")}"></script>')


def determine_css_class(bootstrap_style, css_class):
    if css_class:
        return css_class
    return 'btn ' + ' '.join(['btn-' + s.strip() for s in bootstrap_style.split(',')])


@register.simple_tag
def button_javascript(button_name, url_name=None, url_args=None, **kwargs):
    json_data = {'data': dict(button=button_name, **kwargs)}
    if url_name:
        json_data['url'] = reverse(url_name, args=url_args)
    return f'ajax_helpers.post_json({json.dumps(json_data)})'


@register.simple_tag
def ajax_button(text, name, bootstrap_style='primary', css_class=None, **kwargs):
    return mark_safe(f'''<button class="{determine_css_class(bootstrap_style, css_class) }"''' 
                     f'''onclick='{button_javascript(name, **kwargs) }'>{ text }</button>''')


@register.simple_tag
def send_form(form_id, **kwargs):
    kwargs.update({"form_id": form_id})
    return f'''ajax_helpers.send_form("{form_id}", {json.dumps(kwargs)})'''


@register.simple_tag
def send_form_button(text, form_id, bootstrap_style='primary', css_class=None, **kwargs):
    return mark_safe(f'''<button class="{ determine_css_class(bootstrap_style, css_class) }"''' 
                     f'''onclick='{send_form(form_id, **kwargs) }'>{ text }</button>''')


@register.inclusion_tag('ajax_helpers/upload_file.html')
def upload_file(text='Upload File', bootstrap_style='primary', css_class=None):
    return {
        'id': random_string(),
        'text': text,
        'css_class': determine_css_class(bootstrap_style, css_class),
    }


@register.simple_tag
def tooltip_init(element_id, url_name, css_class='ajax-tooltip'):
    return mark_safe(
        f'''<script>ajax_helpers.tooltip.init('{element_id}', "{reverse(url_name)}", "{css_class}")</script>'''
    )


@register.inclusion_tag('ajax_helpers/ajax_timer.html')
def ajax_timer(name, interval_ms):
    return {
        'name': name,
        'interval': interval_ms
    }



import setuptools

with open("readme.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-ajax-helpers",
    version="0.0.4",
    author="Ian Jones",
    description="Django app to assist with posting Ajax and receiving commands from django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jonesim/ajax-helpers",
    include_package_data = True,
    packages=['ajax_helpers'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
